local _, ns = ...
local WK = ns.WhoKnows
local print = WK.print
local debugPrint = WK.debugPrint
-------------------------------------------------------

WK.ItemStats = {}
WK.Itemcache = {}
WK.Itemcache.ItemIds = {}
WK.Itemcache.ItemFrames = {}
WK.Itemcache.interval = 3
WK.Itemcache.timerON = false

function WK.Itemcache:Add(itemId, interval)
	interval = interval or WK.Itemcache.interval
	if not self.ItemIds[itemId] then
		self.ItemIds[itemId] = true
		debugPrint("Adding " .. itemId .. " to WhoKnows.Itemcache")
	end
	if not WK.Itemcache.timerON then
		debugPrint("making timer...")
		WK.Itemcache.timerON = true
		C_Timer.After(interval, WK.Itemcache.cTimer)
	end
end

function WK.Itemcache:Count()
	local c = 0
	for _ in pairs(self.ItemIds) do
		c = c + 1
	end
	return c
end

function WK.Itemcache.cTimer()
	local ic = WK.Itemcache
	if ic:Count() == 0 then return end
	debugPrint("looking up items in Itemcache")
	for k, v in pairs(ic.ItemIds) do
		ic.ItemIds[k] = nil -- remove the itemId from our table. If it still isn't ready then it will be re-added
		WK.items[k] = nil --clears whatever we have in WhoKnows.items for that item
		if type(v) ~= "boolean" then
			WK.ItemGlow:OverlaySetup(v, k) -- forces a new lookup - if good then the overlay will update if not then it will end up back here.
--		else
--			local x = WK.items[k]
		end
		ic.timerON = false
	end
end

local function SetEnchantSkillGroup(spellName)
	local searchStrings = {
		["oils"] = "%soil$",
		["wands"] = "%swand$",
		["shields"] = "shield",
		["off-hands"] = "off%-hand",
		["boots"] = "enchant%sboots",
		["bracers"] = "enchant%sbracer",
		["chest"] = "enchant%schest",
		["cloak"] = "enchant%scloak",
		["gloves"] = "enchant%sgloves",
		["neck"] = "enchant%sneck",
		["rings"] = "enchant%ring",
		["weapons"] = "weapon",
		["staff"] = "enchant%staff",
	}
	--materials ?
	local catagory
	for k, v in pairs(searchStrings) do
		if spellName:match(v) then
			catagory = k
			break
		end
	end
	if catagory == "off-hands" or catagory == "shields" then
		catagory = "shields and off-hands"
	end
	catagory = catagory or "materials"
	return catagory
end

WK.items = setmetatable({}, {
	__index = function(t, itemId)
		local item = WK.ItemStats:GetItemInfo(itemId)
		if not item then return end
		rawset(t, itemId, item)
		table.sort(t)
		return item
	end,
})

function WK.ItemStats:GetItemInfo(item)
	local itemId
	if type(item) == "number" then
		itemId = item
	elseif type(item) == "string" and WK.IsItemLink(item) then
		itemId = WK.GetItemIdFromHyperlink(item)
	end
	assert(itemId, "Error: WhoKnows.ItemStats:GetItemInfo did not receive a valid Item ID or Item link\n(itemID = " .. (tostring(itemId) or "nil") .. ")")

	local I = {}
	I.itemName, I.itemLink, _, _, _, I.itemClass, I.profession = GetItemInfo(itemId)
	if not I.itemName then debugPrint("itemName is null from GetItemInfo(" .. itemId .. ")") end
	if (I.itemClass ~= "Recipe" and I.itemClass ~= "Miscellaneous") then return end

	I.itemName = I.itemName:gsub("^.*:%s", "") --clean up itemName  - if colon present, remove it and everything preceding it
	I.itemName = I.itemName:gsub("Transmute", "Transmute:") -- spells have a colon after 'Transmute', but not in recipe name, so add it.

	I.itemName = I.itemName:lower()
	I.itemId = itemId
	I.isNotLearnableYet = nil
	I.isBOP = nil
	I.isBOPspell = nil --TODO: the spell itself is BOP 
	I.isBOPItem = nil -- TODO: the item the spell makes is BOP
	I.isKnownBySelf = nil
	I.isBlueprint = nil
	I.reqToonLvl = nil
	I.reqSkillLvl = nil
	I.reqEngiSpec = nil
	I.needsPrereq = nil
	I.enchantCatagory = nil
	I.isIncomplete = false

	local ttLines = WK.ToolTips:EnumerateTooltipLines(I.itemLink) --TODO: need to pass?
	if not ttLines then
		debugPrint("Item " .. itemId .. ", " .. I.itemName .. " not ready - skipped")
		I.isIncomplete = true
		WK.Itemcache:Add(itemId)
		return
	end
	for i = 1, #ttLines do
		if ttLines[i]:match("Retrieving item information") then
			I.isIncomplete = true
			WK.Itemcache:Add(itemId)
		end
		I.isBOP = I.isBOP or (i <= 4 and (ttLines[i]:match("Binds when picked up") or ttLines[i]:match("Soulbound")))
		I.isBOPItem = I.isBOPItem or (i >= 5 and (ttLines[i]:match("Binds when picked up") or ttLines[i]:match("Binds when equipped")))
		I.isKnownBySelf = I.isKnownBySelf or ttLines[i]:match("^Already known")
		I.isBlueprint = I.isBlueprint or ttLines[i]:match("Garrison Blueprint")
		I.reqToonLvl = I.reqToonLvl or ttLines[i]:match('^Requires%sLevel%s(.*)')
		I.reqSkillLvl = I.reqSkillLvl or ttLines[i]:match("Requires%s" .. I.profession .. "%s[(](%d*)[)]")
		I.reqEngiSpec = I.reqEngiSpec or ttLines[i]:match("Requires%s(%w*)%sEngineer")
		I.needsPrereq = I.needsPrereq or (ttLines[i]:match("Must have learned") or ttLines[i]:match("^Requires%sWay%sof"))
		local reqClass = reqClass or ttLines[i]:match('^Classes:%s(.*)')
		if reqClass and not I.reqClass then
			reqClass = reqClass:gsub(",%s", ",")
			I.reqClass = { strsplit(",", reqClass) }
		end
		--track the # of times we loop since if a spell is BOP it will be listed within the 1st three lines but if the item the spell
		-- produces is BOP it will be listed as such later in the tooltip
		i = i + 1
	end
	if I.profession == "Enchanting" then
		I.enchantCatagory = SetEnchantSkillGroup(I.itemName)
		local newItemName = (I.itemName:gsub("^.*%s%-%s", ""))
		I.itemName = newItemName .. "|" .. I.enchantCatagory
	end

	I.reqSkillLvl = tonumber(I.reqSkillLvl) or -1
	I.reqToonLvl = tonumber(I.reqToonLvl) or -1
	if I.isKnownBySelf then I.isKnownBySelf = true end
	return I
end