local _, ns = ...
local WK = ns.WhoKnows
local _VERSION = tonumber(GetAddOnMetadata('WhoKnows', 'version'))
local print = WK.print
local debugPrint = WK.debugPrint
local error = WK.error

-------------------------------------------------------
WK.WKEvFrame:RegisterEvent("VARIABLES_LOADED")

function WK:EnableAddon(enable)
	if (enable) then
		WK.WKEvFrame:RegisterEvent("ADDON_LOADED")
		WK.WKEvFrame:RegisterEvent("CHAT_MSG_SKILL")
		WK.WKEvFrame:RegisterEvent("TRADE_SKILL_SHOW")
		WK.WKEvFrame:RegisterEvent("PLAYER_LEVEL_UP")
		WK.WKEvFrame:RegisterEvent("CHAT_MSG_SYSTEM")
		print("WhoKnows functionality ENABLED")
	else
		WK.WKEvFrame:UnregisterEvent("ADDON_LOADED")
		WK.WKEvFrame:UnregisterEvent("CHAT_MSG_SKILL")
		WK.WKEvFrame:UnregisterEvent("TRADE_SKILL_SHOW")
		WK.WKEvFrame:UnregisterEvent("PLAYER_LEVEL_UP")
		WK.WKEvFrame:UnregisterEvent("CHAT_MSG_SYSTEM")
		print("WhoKnows functionality DISABLED")
	end
end

local function eventHandler(self, event, ...)
	local arg1, arg2, arg3, arg4 = ...
	local debugEvent = false
	if event == "ADDON_LOADED" and arg1 == "Blizzard_AuctionUI" then
		WK.InventoryFrames:AuctionScanEnable(WhoKnows_Data.enableAddon and WhoKnows_Data.ScanEnableAH)
	elseif (event == "VARIABLES_LOADED") then
		LibStub("AceConfig-3.0"):RegisterOptionsTable("Who Knows", WK.options)
		self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Who Knows")
		
		WK.currentPlayer = WK.Character:New()
		WK.db:LoadSettings()
		WK.ToolTips:HookToolTips()
		if WhoKnows_Data and (WhoKnows_Data.version < _VERSION) then WK.db:UpdateDB() end
		if WhoKnows_Data.NeedsFactionUpdate then
			print([[|cffc41f3bWarning:|r this addon will be disabled until you complete the faction selection.
				Type /wk setfactions in chat box to try again.]])
			WhoKnows_Data.enableAddon = false
			return
		end
		WK.PlayerStats:GetPlayerSkillLvls()
		WK.InventoryFrames:LoadItemGlows()
		

		print("Version: " .. _VERSION .. " loaded. Written by Lamecooter, Hydraxis Alliance")
		print("type /wk in the chat window to get a list of options.")
		print("Please report any problems at http://wow.curseforge.com/addons/who-knows/tickets")
		WK:EnableAddon(WhoKnows_Data.enableAddon)
	elseif event == "CHAT_MSG_SKILL" then
		WK.PlayerStats:GetPlayerSkillLvls()
		debugEvent = true
	elseif event == "TRADE_SKILL_SHOW" then
		debugEvent = true
		WK.WKEvFrame:RegisterEvent("TRADE_SKILL_DATA_SOURCE_CHANGED")
		--C_Timer.After(1,function() WK.PlayerStats:GetTradeSkills() end)
	elseif event == "TRADE_SKILL_LIST_UPDATE" then
		debugEvent = true
--		WK.PlayerStats:GetTradeSkills()

	elseif event == "TRADE_SKILL_DATA_SOURCE_CHANGED" then
		--Note: Need to wait until this fires so we'll have something to scan. Then register
		-- TRADE_SKILL_LIST_UPDATE but it fires 3 times in a rowm so we'll unhook it after it runs.
		--WK.WKEvFrame:RegisterEvent("TRADE_SKILL_LIST_UPDATE")
		WK.PlayerStats:GetTradeSkills()
	elseif event == "MERCHANT_SHOW" then WK.InventoryFrames:MerchantScanUpdate(); debugEvent = true;
	elseif event == "MERCHANT_UPDATE" then
		WK.InventoryFrames:MerchantScanUpdate()
	elseif event == "PLAYER_LEVEL_UP" then
		debugEvent = true
		WK.currentPlayer.level = tonumber(arg1)
		local realm = GetRealmName()
		local faction = UnitFactionGroup("player")
		local name = GetUnitName("player")
		WhoKnows_Data.Skills[realm][faction][name]["level"] = WK.currentPlayer.level
		debugprint("Set player level to " .. WhoKnows_Data.Skills[realm][faction][name]["level"])
	elseif event == "GUILDBANKBAGSLOTS_CHANGED" or event == "GUILDBANKFRAME_OPENED" then
		WK.InventoryFrames:GuildBankScanUpdate(self)
	elseif event == "BANKFRAME_OPENED" or event == "PLAYERBANKSLOTS_CHANGED" then
		WK.InventoryFrames:BankScanUpdate(self)
	elseif event == "LOOT_OPENED" or event == "LOOT_SLOT_CLEARED" or event == "LOOT_SLOT_CHANGED" then
		WK.InventoryFrames:LootUpdate(self)
	elseif event == "AUCTION_HOUSE_SHOW" or event == "AUCTION_ITEM_LIST_UPDATE" then
		WK.InventoryFrames:AuctionScanUpdate()
		debugEvent = true
	elseif event == "MAIL_SHOW" or event == "MAIL_SEND_INFO_UPDATE" or event == "MAIL_SEND_SUCCESS" then
		WK.InventoryFrames:ProcessSendMailbox()
	elseif event == "CHAT_MSG_SYSTEM" then
		local newSkill = arg1:match("You have learned how to %a*:(.*)%.")
		if newSkill then 
			debugPrint("OK trying to add ".. (newSkill or "nil") .. " to DB")
			debugEvent = true
			WhoKnows.PlayerStats:UpdateLearnedSkill(newSkill)
		end
	end
	if (debugEvent == true) then
		WK.debugPrint("Event", event, { ... })
	end
end

WK.WKEvFrame:SetScript("OnEvent", eventHandler)

SLASH_WKTOGGLE1 = "/wk"
SlashCmdList["WKTOGGLE"] = function(val)
	if val == "reset" then
		print("This will reset Who Knows data for this toon. If you are sure you want to do this then")
		print("type in |cff00ffff/wk reset true|r. You will have to reload each profession to repopulate the data.")
	elseif val == "reset true" then
		print("Erasing all saved information for " .. WK.currentPlayer.name)
		local playerName = WK.currentPlayer.name
		WhoKnows_Data.Skills[WK.currentPlayer.realm][WK.currentPlayer.faction][playerName] = nil
		print("All WhoKnows data for " .. playerName .. " deleted. Restart or Reload WoW to start collecting information again.")
	elseif val == "reset all" then
		print("This will reset ALL WhoKnows data. If you are sure you want to do this then")
		print("type in |cff00ffff/wk reset all true|r. You will have to reload each of your toons and their professions to repopulate the data.")
	elseif val == "reset all true" then
		print("Erasing all saved information.")
		WhoKnows_Data = nil
		print("All WhoKnows data deleted. Restart or Reload WoW to start collecting information again.")
	elseif string.find(val, "delete") then
		local _, toon = strsplit(" ", val)
		toon = (toon:gsub("^%l", string.upper))
		if toon and WhoKnows_Data.Skills[WK.currentPlayer.realm][WK.currentPlayer.faction][toon] then
			WhoKnows_Data.Skills[WK.currentPlayer.realm][WK.currentPlayer.faction][toon] = nil
			print("The toon " .. toon .. " was deleted from WhoKnows.")
		elseif toon and WhoKnows_Data.Skills[WK.currentPlayer.realm][WK.currentPlayer.faction][toon] == nil then
			print("Don't have a toon named " .. toon .. " to delete.")
		end
	elseif val == "setfactions" then
		print("loading faction selection frame")
		WK.wkframes:AssignFaction()
	elseif val == "debug" then
		WhoKnows_Data.enableDebug = not WhoKnows_Data.enableDebug
		WK.debug = WhoKnows_Data.enableDebug
		if (WK.debug) then
			print("Debug: On")
		else
			print("Debug: Off")
		end
	elseif val == "config" then
		InterfaceOptionsFrame_OpenToCategory("Who Knows")
	elseif val == "disable" then
		WK:EnableAddon(false)
	elseif val == "enable" then
		WK:EnableAddon(true)
	elseif val == "status" or val == "ver" then
		print("Version loaded: " .. tostring(_VERSION))
		print("Memory Usage: " .. GetAddOnMemoryUsage("WhoKnows") .. "kb")
	elseif val == "tut" then
		WK.wkframes:Tutorial() --in development
	elseif val == "intro" then
		local F = WK.wkframes:IntroDialog()
		F:Show()
	elseif val == "itemclear" then
		local c = WK:tlength(WK.items)
		table.wipe(WK.items)
		local cc = WK:tlength(WK.items)
		debugPrint(string.format("WhoKnows.items emptied of %s items. Now it has %s", c, cc))
		table.wipe(WK.DebugTable.item)
	else
		print("Command: reset - Deletes current toon's saved settings")
		print("Command: reset all - Deletes EVERY toon's settings")
		print("Command: delete [toon name] - Deletes named toon's settings. Example: /wk delete Jenkins")
		print("Command: debug - toggle verbose output on or off")
		print("Command: config - Open this Addon's configuration options")
		print("Command: enable - Enable WhoKnows")
		print("Command: disable - Disable WhoKnows")
		print("command: status or ver: - Display version information")
	end
end

_G.WhoKnows = WK
