local _, ns = ...
local WK = ns.WhoKnows
-------------------------------------------------------

WK.options = {
	name = "Who Knows",
	handler = WhoKnows,
	type = "group",
	childGroups = "tab",
	args = {
		addonNotes = {
			type = "description",
			name = "Note: these settings are account wide.",
			width = "full",
			order = 1,
		},
	}
}

WK.options.args.colors = {
	type = "group",
	name = "Colors",
	order = 2,
	disabled = function() return not WhoKnows_Data.enableAddon end,
	args = {
		enableSelfLearnableColor = {
			type = "toggle",
			name = "Enable",
			desc = "",
			width = "half",
			descStyle = "inline",
			order = 2,
			get = function(info)
				return WhoKnows_Data.enableSelfLearnableColor
			end,
			set = function(info, value)
				WhoKnows_Data.enableSelfLearnableColor = value
			end,
		},
		selfLearnableColor = {
			type = "color",
			name = "Color of learnable item",
			desc = "Sets the border color of items that the current character is able to learn.",
			width = "double",
			order = 3,
			get = function(info)
				local c = WhoKnows_Data.Colors.SelfLearnableColor
				return c[1], c[2], c[3], c[4]
			end,
			set = function(info, r, g, b, a)
				local c = WhoKnows_Data.Colors.SelfLearnableColor
				c[1] = r
				c[2] = g
				c[3] = b
				c[4] = a
			end,
		},
		resetSelfLearnableColor = {
			type = "execute",
			name = "Reset",
			width = "half",
			order = 4,
			func = function()
				WhoKnows_Data.Colors.SelfLearnableColor = { 0.12, 1, 0, 1 }
			end,
		},
		enableAltLearnableColor = {
			type = "toggle",
			name = "Enable",
			width = "half",
			descStyle = "inline",
			order = 5,
			get = function(info)
				return WhoKnows_Data.enableAltLearnableColor
			end,
			set = function(info, value)
				WhoKnows_Data.enableAltLearnableColor = value
			end,
		},
		altLearnableColor = {
			type = "color",
			name = "Color of learnable item for alt characters",
			desc = "Sets the border color of items that an alternate character is able to learn.",
			width = "double",
			order = 6,
			get = function(info)
				local c = WhoKnows_Data.Colors.AltLearnableColor
				return c[1], c[2], c[3], c[4]
			end,
			set = function(info, r, g, b, a)
				local c = WhoKnows_Data.Colors.AltLearnableColor
				c[1] = r
				c[2] = g
				c[3] = b
				c[4] = a
			end,
		},
		resetAltLearnableColor = {
			type = "execute",
			name = "Reset",
			width = "half",
			order = 7,
			func = function()
				WhoKnows_Data.Colors.AltLearnableColor = { 1, 1, 0, 1 }
			end,
		},
		enableAlreadyKnownColor = {
			type = "toggle",
			name = "Enable",
			width = "half",
			descStyle = "inline",
			order = 8,
			get = function(info)
				return WhoKnows_Data.enableAlreadyKnownColor
			end,
			set = function(info, value)
				WhoKnows_Data.enableAlreadyKnownColor = value
			end,
		},
		alreadyKnownColor = {
			type = "color",
			name = "Color of item already known",
			desc = "Sets the border color of items this or an alternate character already knows.",
			width = "double",
			order = 9,
			get = function(info)
				local c = WhoKnows_Data.Colors.AlreadyKnownColor
				return c[1], c[2], c[3], c[4]
			end,
			set = function(info, r, g, b, a)
				local c = WhoKnows_Data.Colors.AlreadyKnownColor
				c[1] = r
				c[2] = g
				c[3] = b
				c[4] = a
			end,
		},
		resetAlreadyKnownColor = {
			type = "execute",
			name = "Reset",
			width = "half",
			order = 10,
			func = function()
				WhoKnows_Data.Colors.AlreadyKnownColor = { 0.12, 0.39, 0.77, 1 }
			end,
		},
		enableNotReadyToLearnColor = {
			type = "toggle",
			name = "Enable",
			width = "half",
			descStyle = "inline",
			order = 11,
			get = function(info)
				return WhoKnows_Data.enableNotReadyToLearnColor
			end,
			set = function(info, value)
				WhoKnows_Data.enableNotReadyToLearnColor = value
			end,
		},
		notReadyToLearnColor = {
			type = "color",
			name = "Color of item not ready to learn",
			desc = "Sets the border color of items that this or an alternate character can't learn yet.",
			width = "double",
			order = 12,
			get = function(info)
				local c = WhoKnows_Data.Colors.NotReadyToLearnColor
				return c[1], c[2], c[3], c[4]
			end,
			set = function(info, r, g, b, a)
				local c = WhoKnows_Data.Colors.NotReadyToLearnColor
				c[1] = r
				c[2] = g
				c[3] = b
				c[4] = a
			end,
		},
		resetNotReadyToLearnColor = {
			type = "execute",
			name = "Reset",
			width = "half",
			order = 13,
			func = function()
				WhoKnows_Data.Colors.NotReadyToLearnColor = { 1, 0.5, 0, 1 }
			end,
		},
		enableUnlearnableColor = {
			type = "toggle",
			name = "Enable",
			width = "half",
			descStyle = "inline",
			order = 14,
			get = function(info)
				return WhoKnows_Data.enableUnlearnableColor
			end,
			set = function(info, value)
				WhoKnows_Data.enableUnlearnableColor = value
			end,
		},
		unlearnableColor = {
			type = "color",
			name = "Color of unlearnable item",
			desc = "Sets the border color of items that none of your characters can learn. :(",
			width = "double",
			order = 15,
			get = function(info)
				local c = WhoKnows_Data.Colors.UnlearnableColor
				return c[1], c[2], c[3], c[4]
			end,
			set = function(info, r, g, b, a)
				local c = WhoKnows_Data.Colors.UnlearnableColor
				c[1] = r
				c[2] = g
				c[3] = b
				c[4] = a
			end,
		},
		resetUnlearnableColor = {
			type = "execute",
			name = "Reset",
			width = "half",
			order = 16,
			func = function()
				WhoKnows_Data.Colors.UnlearnableColor = { 0.768, 0.125, 0.231, 1 }
			end,
		},
	},
}

WK.options.args.switches = {
	type = "group",
	name = "On/Off",
	order = 1,
	args = {
		enableAddon = {
			type = "toggle",
			name = "Enable Addon",
			desc = "Toggles the Addon on or off.",
			width = "full",
			order = 1,
			disabled = function()
				return WhoKnows_Data.NeedsFactionUpdate
			end,
			get = function(info)
				return WhoKnows_Data.enableAddon
			end,
			set = function(info, value)
				WhoKnows_Data.enableAddon = value
				WhoKnows:EnableAddon(value)
			end,
		},
		enableDebug = {
			type = "toggle",
			name = "Debug",
			desc = "Toggles Debug on or off.",
			width = "full",
			order = 1,
			get = function(info)
				return WhoKnows_Data.enableDebug
			end,
			set = function(info, value)
				WhoKnows_Data.enableDebug = value
				WhoKnows.debug = WhoKnows_Data.enableDebug;
			end,
		},
		toggleBagScan = {
			type = "toggle",
			name = "Enable scanning of Bags",
			width = "full",
			disabled = function()
				return not WhoKnows_Data.enableAddon
			end,
			get = function(info)
				return WhoKnows_Data.ScanEnableBag
			end,
			set = function(info, value)
				WhoKnows_Data.ScanEnableBag = value
				WhoKnows.InventoryFrames:BagScanEnable(value)
			end,
		},
		toggleBankScan = {
			type = "toggle",
			name = "Enable scanning of the Bank",
			width = "full",
			disabled = function()
				return not WhoKnows_Data.enableAddon
			end,
			get = function(info)
				return WhoKnows_Data.ScanEnableBank
			end,
			set = function(info, value)
				WhoKnows_Data.ScanEnableBank = value;
				WhoKnows.InventoryFrames:BankScanEnable(value)
			end,
		},
		toggleGuildBankScan = {
			type = "toggle",
			name = "Enable scanning of the Guild Bank",
			width = "full",
			disabled = function()
				return not WhoKnows_Data.enableAddon
			end,
			get = function(info)
				return WhoKnows_Data.ScanEnableGuildBank
			end,
			set = function(info, value)
				WhoKnows_Data.ScanEnableGuildBank = value;
				WhoKnows.InventoryFrames:GuildBankScanEnable(value)
			end,
		},
		toggleVendorScan = {
			type = "toggle",
			name = "Enable scanning of Vendors",
			width = "full",
			disabled = function()
				return not WhoKnows_Data.enableAddon
			end,
			get = function(info)
				return WhoKnows_Data.ScanEnableVendors
			end,
			set = function(info, value)
				WhoKnows_Data.ScanEnableVendors = value;
				WhoKnows.InventoryFrames:MerchantScanEnable(value)
			end,
		},
		toggleLootScan = {
			type = "toggle",
			name = "Enable scanning of the Loot Window",
			width = "full",
			disabled = function()
				return not WhoKnows_Data.enableAddon
			end,
			get = function(info)
				return WhoKnows_Data.ScanEnableLoot
			end,
			set = function(info, value)
				WhoKnows_Data.ScanEnableLoot = value;
				WhoKnows.InventoryFrames:LootEnable(value)
			end,
		},
		toggleAuctionScan = {
			type = "toggle",
			name = "Enable scanning of the Auction House",
			width = "full",
			disabled = function()
				return not WhoKnows_Data.enableAddon --true
			end,
			get = function(info)
				return WhoKnows_Data.ScanEnableAH
			end,
			set = function(info, value)
				WhoKnows_Data.ScanEnableAH = value;
				WhoKnows.InventoryFrames:AuctionScanEnable(value)
			end,
		},
		toggleMailBoxScanScan = {
			type = "toggle",
			name = "Enable scanning of the Mailbox",
			width = "full",
			disabled = function()
				return not WhoKnows_Data.enableAddon
			end,
			get = function(info)
				return WhoKnows_Data.ScanEnableMailbox
			end,
			set = function(info, value)
				WhoKnows_Data.ScanEnableMailbox = value;
				WhoKnows.InventoryFrames:MailboxScanEnable(WhoKnows_Data.ScanEnableMailbox)
			end,
		},
		toggleRecipeCheck = {
			type = "toggle",
			name = "Enable checking of Recipes",
			width = "full",
			disabled = function()
				return not WhoKnows_Data.enableAddon
			end,
			get = function(info)
				return WhoKnows_Data.EnableRecipeCheck
			end,
			set = function(info, value)
				WhoKnows_Data.EnableRecipeCheck = value
			end,
		},
		toggleGarrisonBlueprintCheck = {
			type = "toggle",
			name = "Enable checking of Garrison Blueprints",
			width = "full",
			disabled = function()
				return not WhoKnows_Data.enableAddon
			end,
			get = function(info)
				return WhoKnows_Data.GarrisonBlueprintCheck
			end,
			set = function(info, value)
				WhoKnows_Data.GarrisonBlueprintCheck = value
			end,
		},
	},
}

WK.options.args.excludePlayers = {
	type = "group",
	name = "Player Exclusion",
	order = 3,
	disabled = function() return not WhoKnows_Data.enableAddon end,
	args = {
		list = {
			name = "Toons to |cffffff00Exclude|r from scanning (bankers, etc.)",
			type = "multiselect",
			width = "double",
			tristate = false,
			values = function()
				WhoKnows.PlayerList = WhoKnows.PlayerList or WK.Character:GetAllCharacters()
				local t = {}
				for _, p in ipairs(WhoKnows.PlayerList) do
					local f = (p[2] == "Horde") and "|cffc41f3bHorde|r" or "|cff0070deAlliance|r"
					local n = "|cffffff00" .. p[3] .. "|r"
					table.insert(t, p[1] .. ": " .. f .. " :" .. n)
				end
				return t
			end,
			get = function(info, key)
				r, f, n = unpack(WhoKnows.PlayerList[key])
				return WhoKnows_Data["Skills"][r][f][n]["excludeFromScan"] or false
			end,
			set = function(info, key, value)
				r, f, n = unpack(WhoKnows.PlayerList[key])
				WhoKnows_Data["Skills"][r][f][n]["excludeFromScan"] = value
			end,
		}
	}
}